/*
 * This file is a part of VyQChat.
 *
 * Copyright (C) 2002-2003 Pawel Stolowski <yogin@linux.bydg.org>
 *
 * VyQChat is free software; you can redestribute it and/or modify it
 * under terms of GNU General Public License by Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY. See GPL for more details.
 */

#include "config.h"
#include "settings.h"
#include <stdlib.h>
#include <qsettings.h>
#include <qlayout.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qtooltip.h>
#include <qvalidator.h>
#include <qhostaddress.h>
#include <qradiobutton.h>
#include <qvbox.h>
#include <qvbuttongroup.h>
#include <qpushbutton.h>
#include <qfontdialog.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <arpa/inet.h>

#include <qtextcodec.h>
#include <qfiledialog.h>

#ifndef DATADIR
#define DATADIR "./"
#endif

extern bool amIroot();

Settings::Settings()/*{{{*/
{
}/*}}}*/

Settings::~Settings()/*{{{*/
{
	save();
}/*}}}*/

bool Settings::getDefaults()/*{{{*/
{
	QString datapath(DATADIR);

	bool status = false;
	def_topic = "no topic";
	def_sound = Sound::PCSpeaker;
	def_playcmd = "/usr/bin/play";
	def_snd[Sound::Chatline] = datapath + "chatline.wav";
	def_snd[Sound::Beep] = datapath + "beep.wav";
	def_snd[Sound::JoinChannel] = datapath + "join.wav";
	def_snd[Sound::LeaveChannel] = datapath + "leave.wav";
	def_snd[Sound::Message] = datapath + "beep.wav";
	def_snd[Sound::RegexpMatch] = datapath + "beep.wav";

	//
	// get hostname
	char tmp[255];
	gethostname(tmp, 254);
	def_hostname = QString(tmp);

	//
	// get username
	char *login = getenv("LOGNAME");
	def_nick = QString(login);
	
	//
	// get eth0 parameters
	def_broadcast = "127.255.255.255";
	def_ip = "127.0.0.1";

	struct ifconf ifc;
	int sockfd = socket(AF_INET, SOCK_DGRAM, 0);

	ifc.ifc_req = new ifreq [16];
	ifc.ifc_len = 16*sizeof(struct ifreq);

	if (ioctl(sockfd, SIOCGIFCONF, &ifc)==0)
	{
		status = true;

		const int num = ifc.ifc_len/sizeof(struct ifreq);
		struct ifreq *req = ifc.ifc_req; 
		for (int i=0; i<num; i++)
		{
			if (strcmp(req[i].ifr_name, "eth0")==0)
			{
				if (ioctl(sockfd, SIOCGIFBRDADDR , &req[i])==0) 
					def_broadcast = QString(inet_ntoa(*(in_addr *)(&req[i].ifr_broadaddr.sa_data[2])));
				else
					status = false;
				
				if (ioctl(sockfd, SIOCGIFADDR , &req[i])==0) 
					def_ip = QString(inet_ntoa(*(in_addr *)(&req[i].ifr_addr.sa_data[2])));
				else
					status = false;
			}
		}
		
	}
	delete [] ifc.ifc_req;
	close(sockfd);
	return status;
}/*}}}*/

void Settings::load()/*{{{*/
{
	getDefaults();

	QSettings cfg;
	cfg.beginGroup("/vyqchat");

	cfg.beginGroup("/Network");
		nick = cfg.readEntry("/Nick", def_nick);
		hostname = cfg.readEntry("/Hostname", def_hostname);
		ip = cfg.readEntry("/IP", def_ip);
		broadcast = cfg.readEntry("/Broadcast", def_broadcast);
		refresh = cfg.readNumEntry("/Refresh", 300);
	cfg.endGroup();
	cfg.beginGroup("/Misc");
		encoding = cfg.readEntry("/Encoding", "");
		hide_channels = cfg.readBoolEntry("/HideChannels", false);
		popup = cfg.readBoolEntry("/Popup", true);
		nopopup_dnd = cfg.readBoolEntry("/NoPopupDND", true);
	cfg.endGroup();
	cfg.beginGroup("/General");
		regexp_on = cfg.readBoolEntry("/RegExpOn", false);
		toolbar_on = cfg.readBoolEntry("/ToolbarOn", true);
		userslist_on = cfg.readBoolEntry("/UsersListOn", true);
		topic = cfg.readEntry("/Topic", def_topic);
	cfg.endGroup();
	cfg.beginGroup("/Sound");
		sound = static_cast<Sound::SoundType>(cfg.readNumEntry("/sound", def_sound));
		playcmd = cfg.readEntry("/PlayCommand", def_playcmd);
	cfg.endGroup();
	cfg.beginGroup("/Wavs");
		snd[Sound::Chatline] = cfg.readEntry("/Chatline", def_snd[Sound::Chatline]);
		snd[Sound::Beep] = cfg.readEntry("/Beep", def_snd[Sound::Beep]);
		snd[Sound::Message] = cfg.readEntry("/Message", def_snd[Sound::Message]);
		snd[Sound::JoinChannel] = cfg.readEntry("/Join", def_snd[Sound::JoinChannel]);
		snd[Sound::LeaveChannel] = cfg.readEntry("/Leave", def_snd[Sound::LeaveChannel]);
		snd[Sound::RegexpMatch] = cfg.readEntry("/RegexpMatch", def_snd[Sound::RegexpMatch]);
		sndp[Sound::Chatline] = cfg.readBoolEntry("/pChatline", false);
		sndp[Sound::Beep] = cfg.readBoolEntry("/pBeep", false);
		sndp[Sound::Message] = cfg.readBoolEntry("/pMessage", false);
		sndp[Sound::JoinChannel] = cfg.readBoolEntry("/pJoin", false);
		sndp[Sound::LeaveChannel] = cfg.readBoolEntry("/pLeave", false);
		sndp[Sound::RegexpMatch] = cfg.readBoolEntry("/pRegexpMatch", false);
	cfg.endGroup();
	cfg.beginGroup("/Window");
		x = cfg.readNumEntry("/x", -1);
		y = cfg.readNumEntry("/y", -1);
		width = cfg.readNumEntry("/Width", 480);
		height = cfg.readNumEntry("/Height", 360);
		listwidth = cfg.readNumEntry("/ListWidth", 120);
	cfg.endGroup();

	cfg.endGroup();
}/*}}}*/

void Settings::save()/*{{{*/
{	
	if (amIroot())
		return;
	QSettings cfg;
	cfg.beginGroup("/vyqchat");

	cfg.beginGroup("/Network");
		cfg.writeEntry("/Nick", nick);
		cfg.writeEntry("/Hostname", hostname);
		cfg.writeEntry("/IP", ip);
		cfg.writeEntry("/Broadcast", broadcast);
		cfg.writeEntry("/Refresh", refresh);
	cfg.endGroup();
	cfg.beginGroup("/Misc");
		cfg.writeEntry("/Encoding", encoding);
		cfg.writeEntry("/HideChannels", hide_channels);
		cfg.writeEntry("/Popup", popup);
		cfg.writeEntry("/NoPopupDND", nopopup_dnd);
	cfg.endGroup();
	cfg.beginGroup("/General");
		cfg.writeEntry("/RegExpOn", regexp_on);
		cfg.writeEntry("/ToolbarOn", toolbar_on);
		cfg.writeEntry("/UsersListOn", userslist_on);
		cfg.writeEntry("/Topic", topic);
	cfg.endGroup();
	cfg.beginGroup("/Sound");
		cfg.writeEntry("/sound", static_cast<int>(sound));
		cfg.writeEntry("/PlayCommand", playcmd);
	cfg.endGroup();
	cfg.beginGroup("/Wavs");
		cfg.writeEntry("/Chatline", snd[Sound::Chatline]);
		cfg.writeEntry("/Join", snd[Sound::JoinChannel]);
		cfg.writeEntry("/Leave", snd[Sound::LeaveChannel]);
		cfg.writeEntry("/Beep", snd[Sound::Beep]);
		cfg.writeEntry("/Message", snd[Sound::Message]);
		cfg.writeEntry("/RegexpMatch", snd[Sound::RegexpMatch]);
		cfg.writeEntry("/pChatline", sndp[Sound::Chatline]);
		cfg.writeEntry("/pJoin", sndp[Sound::JoinChannel]);
		cfg.writeEntry("/pLeave", sndp[Sound::LeaveChannel]);
		cfg.writeEntry("/pBeep", sndp[Sound::Beep]);
		cfg.writeEntry("/pMessage", sndp[Sound::Message]);
		cfg.writeEntry("/pRegexpMatch", sndp[Sound::RegexpMatch]);
	cfg.endGroup();
	cfg.beginGroup("/Window");
		cfg.writeEntry("/x", x);
		cfg.writeEntry("/y", y);
		cfg.writeEntry("/Width", width);
		cfg.writeEntry("/Height", height);
		cfg.writeEntry("/ListWidth", listwidth);
	cfg.endGroup();
	cfg.endGroup();
}/*}}}*/

SettingsDialog::SettingsDialog(QWidget *parent, Settings *s): QTabDialog(parent, NULL, true), currsettings(s)/*{{{*/
{

	this->settings = *s;
	
	setCaption(tr("VyQchat settings"));

	setupNetworkTab(new QWidget(this));
	setupSoundDeviceTab(new QWidget(this));
	setupSoundsTab(new QWidget(this));
	setupLookTab(new QWidget(this));
	setupMiscTab(new QWidget(this));

	//
	// final setup
	setOkButton();
	setCancelButton();
	setDefaultButton();
	connect(this, SIGNAL(applyButtonPressed()), this, SLOT(slotApply()));
	connect(this, SIGNAL(defaultButtonPressed()), this, SLOT(slotDefaults()));
}/*}}}*/

SettingsDialog::~SettingsDialog()/*{{{*/
{
}
/*}}}*/

void SettingsDialog::setupNetworkTab(QWidget *p)/*{{{*/
{
	QGridLayout *grid1 = new QGridLayout(p, 5, 2, 5);
	grid1->addWidget(new QLabel("Nickname", p), 0, 0);
	grid1->addWidget(le_nick = new QLineEdit(settings.nick, p), 0, 1);
	//grid1->addWidget(new QLabel("Gender", p), 1, 0);
	//grid1->addWidget(le_nick = new QLineEdit(settings.nick, p), 1, 1);
	grid1->addWidget(new QLabel("Hostname", p), 1, 0);
	grid1->addWidget(le_hostname = new QLineEdit(settings.hostname, p), 1, 1);
	grid1->addWidget(new QLabel("IP", p), 2, 0);
	grid1->addWidget(le_ip = new QLineEdit(settings.ip, p), 2, 1);
	grid1->addWidget(new QLabel("Broadcast", p), 3, 0);
	grid1->addWidget(le_broadcast = new QLineEdit(settings.broadcast, p), 3, 1);
	grid1->addWidget(new QLabel("Users-List Refresh", p), 4, 0);
	grid1->addWidget(sp_refresh = new QSpinBox(0, 300, 30, p), 4, 1);
	sp_refresh->setValue(settings.refresh);
	sp_refresh->setMinimumWidth(100);
	sp_refresh->setSuffix(" sec.");
	QRegExp rx("[a-zA-Z]*");
	QRegExpValidator *v_nick = new QRegExpValidator(rx, this);
	le_nick->setValidator(v_nick);
	QToolTip::add(le_ip, tr("IP address of your network\ninterface, e.g. 192.168.0.1"));
	QToolTip::add(le_broadcast, tr("Broadcast address of your network\ninterface, e.g. 192.168.0.255"));
	QToolTip::add(sp_refresh, tr("Update interval for users\nlist (0 for no update)"));

	addTab(p, "Network");
}/*}}}*/

void SettingsDialog::setupSoundDeviceTab(QWidget *p)/*{{{*/
{
	QVBoxLayout *box1 = new QVBoxLayout(p, 5);
	opt_snddev = new QVButtonGroup(p);
	QRadioButton *rb_nosnd = new QRadioButton("No sound", opt_snddev);
	QRadioButton *rb_pcs = new QRadioButton("PC speaker", opt_snddev);
	QRadioButton *rb_arts = new QRadioButton("aRtS", opt_snddev);
	QRadioButton *rb_alsa = new QRadioButton("Alsa", opt_snddev);
	QRadioButton *rb_esd = new QRadioButton("ESound", opt_snddev);
	QRadioButton *rb_cmd = new QRadioButton("External command", opt_snddev);
	
	QHBox *box2 = new QHBox(p);
	new QLabel("Command", box2);
	le_playcmd = new QLineEdit(box2);
	le_playcmd->setText(settings.playcmd);
	le_playcmd->setDisabled(true);

#	ifndef WITH_ARTS
	rb_arts->setDisabled(true);
#	endif
#	ifndef WITH_ALSA
	rb_alsa->setDisabled(true);
#	endif
#	ifndef WITH_ESD
	rb_esd->setDisabled(true);
#	endif
	switch (settings.sound)
	{
		case Sound::NoSound: rb_nosnd->setChecked(true); break;
		case Sound::PCSpeaker: rb_pcs->setChecked(true); break;
		case Sound::Arts: rb_arts->setChecked(true); break;
		case Sound::Esd: rb_esd->setChecked(true); break;
		case Sound::Command: rb_cmd->setChecked(true);
			le_playcmd->setDisabled(false); break;
		default: rb_pcs->setChecked(true); break;
	}
	QToolTip::add(opt_snddev, "Preffered sound output");
	QToolTip::add(le_playcmd, "External command to play sound file.\nMust take file name as last argument");

	connect(opt_snddev, SIGNAL(clicked(int)), this, SLOT(slotSoundSystemClicked(int)));
	box1->addWidget(opt_snddev, 0, 0);
	box1->addWidget(box2, 1, 0);

	addTab(p, "Sound device");
}/*}}}*/

void SettingsDialog::setupSoundsTab(QWidget *p)/*{{{*/
{
	QVBoxLayout *box1 = new QVBoxLayout(p, 5);
	
	snd_list = new QListBox(p);
	snd_list->insertItem("Chat");
	snd_list->insertItem("Message");
	snd_list->insertItem("Join channel");
	snd_list->insertItem("Leave channel");
	snd_list->insertItem("Beep");
	snd_list->insertItem("RegExp match");
	
	box1->addWidget(snd_list);

	QHBox *box2 = new QHBox(p);
	box1->addWidget(box2);
	
	b_play = new QCheckBox("Play:", box2);
	le_sndname = new QLineEdit(box2);
	QPushButton *b_browse = new QPushButton("Browse", box2);

	connect(b_browse, SIGNAL(clicked()), this, SLOT(slotBrowseSoundClicked()));
	connect(b_play, SIGNAL(toggled(bool)), this, SLOT(slotPlayFlagClicked(bool)));
	connect(snd_list, SIGNAL(highlighted(int)), this, SLOT(slotSoundEventSelected(int)));
	snd_list->setSelected(0, true);
	addTab(p, "Sounds");
}/*}}}*/

void SettingsDialog::setupLookTab(QWidget *p)/*{{{*/
{
	p->setDisabled(true);

	const QString change("Set");
	QGridLayout *grid4 = new QGridLayout(p, 4, 3, 5);
	grid4->addWidget(new QLabel("Chat window", p), 0, 0);
	QLineEdit *le_chatfont = new QLineEdit("....", p);
	QPushButton *b_chatfont = new QPushButton(change, p);
	grid4->addWidget(le_chatfont, 0, 1);
	grid4->addWidget(b_chatfont, 0 ,2);
	grid4->addWidget(new QLabel("Users list", p), 1, 0);
	QLineEdit *le_usersfont = new QLineEdit("....", p);
	QPushButton *b_usersfont = new QPushButton(change, p);
	grid4->addWidget(le_usersfont, 1, 1);
	grid4->addWidget(b_usersfont, 1 ,2);

/*	const QString change("Set");
	QWidget *p = new QWidget(this);

	QVGroupBox *group1 = new QVGroupBox("Fonts", p);
	QHBox *box1 = new QHBox(group1);
		
	new QLabel("Chat window", box1);
	QLineEdit *le_chatfont = new QLineEdit("....", box1);
	QPushButton *b_chatfont = new QPushButton(change, box1);*/

	addTab(p, "Look");
}/*}}}*/

void SettingsDialog::setupMiscTab(QWidget *p)/*{{{*/
{
	QVBoxLayout *box = new QVBoxLayout(p, 5);
	QHBox *box1 = new QHBox(p);
	new QLabel("Character encoding", box1);
	box->addWidget(box1);
	
	int selected = 0;
	l_enc = new QComboBox(box1);
	l_enc->insertItem(tr("System default"));
	QTextCodec *codec;
	for (int i=0; codec = QTextCodec::codecForIndex(i); i++)
	{
		//
		// do not allow unicode
		if (strcmp(codec->name(), "UTF-8"))
		{
			l_enc->insertItem(codec->name());
			if (strcmp(codec->name(), settings.encoding.latin1()) == 0)
				selected = i+1;
		}
	}
	l_enc->setCurrentItem(selected);

	b_hide = new QCheckBox("Hide your channels list", p);
	b_hide->setChecked(settings.hide_channels);
	box->addWidget(b_hide);
	b_popup = new QCheckBox("Pop-up windows for messages", p);
	b_popup->setChecked(settings.popup);
	box->addWidget(b_popup);
	b_popupdnd = new QCheckBox("No pop-ups when DND/Away/Offline", p);
	b_popupdnd->setChecked(settings.nopopup_dnd);
	box->addWidget(b_popupdnd);
	slotPopupCheckBoxChanged(settings.popup);

	QToolTip::add(b_hide, "Doesn't allow anyone to see channels\nyou are on when requesting info");
	QToolTip::add(b_popup, "Enables separate windows (pop-ups) for\nmessages that others send to you");
	QToolTip::add(b_popupdnd, "Disables message windows when they would disturb you.\nMessages will still be visible in main chat area");
	
	connect(b_popup, SIGNAL(toggled(bool)), this, SLOT(slotPopupCheckBoxChanged(bool)));
	addTab(p, "Misc");
}/*}}}*/

void SettingsDialog::slotApply()/*{{{*/
{
	settings.nick = le_nick->text();
	settings.hostname = le_hostname->text();
	settings.ip = le_ip->text();
	settings.broadcast = le_broadcast->text();
	settings.refresh = sp_refresh->value();
	settings.sound = (Sound::SoundType)opt_snddev->id(opt_snddev->selected());
	
	settings.encoding = l_enc->currentItem() ? l_enc->currentText() : "";
	settings.hide_channels = b_hide->isChecked();
	settings.popup = b_popup->isChecked();
	settings.nopopup_dnd = b_popupdnd->isChecked();
	
	//
	// update application settings
	*currsettings = settings;
	close();
}/*}}}*/

void SettingsDialog::slotDefaults()/*{{{*/
{
	le_nick->setText(settings.def_nick);
	le_hostname->setText(settings.def_hostname);
	le_ip->setText(settings.def_ip);
	le_broadcast->setText(settings.def_broadcast);
	sp_refresh->setValue(300);
	opt_snddev->setButton(0);
	l_enc->setCurrentItem(0);

}/*}}}*/

/*
 * disables or enables "No popups when DND...." 
 * according to "Popup windows for..." state */
void SettingsDialog::slotPopupCheckBoxChanged(bool state)/*{{{*/
{
	b_popupdnd->setDisabled(!state);
}/*}}}*/

/*
 * mouse-click on sounds-events list
 */
void SettingsDialog::slotSoundEventSelected(int index)/*{{{*/
{
	le_sndname->setText(settings.snd[index]);
	le_sndname->setDisabled(!settings.sndp[index]);
	b_play->setChecked(settings.sndp[index]);
}/*}}}*/

/*
 * opens file dialog for browsing a soundfile
 */
void SettingsDialog::slotBrowseSoundClicked()/*{{{*/
{
	QString start = le_sndname->text();
	if (start.isEmpty())
		start = QString(DATADIR);
	QString selected = QFileDialog::getOpenFileName(start, 
			"Sounds (*.wav);;All files (*.*)", this, 0, "Select sound file");
	if (!selected.isNull())
	{
		le_sndname->setText(selected);
		settings.snd[snd_list->currentItem()] = selected;
	}
}/*}}}*/

/*
 * mouse-click on sound subsystems list
 */
void SettingsDialog::slotSoundSystemClicked(int index)/*{{{*/
{
	le_playcmd->setDisabled(index!=5);
}/*}}}*/

/*
 * toggling of "Play" checkbox
 */
void SettingsDialog::slotPlayFlagClicked(bool state)/*{{{*/
{
	le_sndname->setDisabled(!state);
	settings.sndp[snd_list->currentItem()] = state;
}/*}}}*/

