/*
 * This file is a part of VyQChat.
 *
 * Copyright (C) 2002-2003 Pawel Stolowski <yogin@linux.bydg.org>
 *
 * VyQChat is free software; you can redestribute it and/or modify it
 * under terms of GNU General Public License by Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY. See GPL for more details.
 */

#include "config.h"
#include "sound.h"
#include "settings.h"
#include <qapplication.h>
#include <qfile.h>
#include <qprocess.h>
#include <qglobal.h>

#ifdef WITH_ARTS
#include <artsc.h>
#endif

Sound::SoundType Sound::sndtype;
SoundSystem *Sound::sndsys = NULL;

bool Sound::reload_samples(Settings &settings)/*{{{*/
{
	return sndsys->reload_samples(settings);
}/*}}}*/

bool Sound::init(Settings *settings)/*{{{*/
{
	sndtype = settings->sound;
			
	switch (sndtype)
	{
		case NoSound:
			sndsys = new SoundSystem();
			break;
		case PCSpeaker:
			sndsys = new SoundPCSpeaker();
			break;
		case Command:
			sndsys = new SoundCommand();
			break;
		case Arts:
			sndsys = new SoundArts();
			break;
		case Esd: 
			break;
		case Alsa:
			sndsys = new SoundAlsa();
			break;
		default: return false;
	}
	if (!sndsys)
		return false;
	bool status = sndsys->init();
	if (!status)
	{
		delete sndsys;
		sndsys = new SoundPCSpeaker();
		status = sndsys->init();
	}
	if (status)
		sndsys->reload_samples(*settings);
	return status;
}/*}}}*/

void Sound::close()/*{{{*/
{
	if (sndsys)
		sndsys->close();
}/*}}}*/

void Sound::play(Sfx sfx)/*{{{*/
{
	if (sndsys)
		sndsys->play(sfx);
}/*}}}*/

const QString& Sound::error()/*{{{*/
{
	return sndsys->errstr;
}/*}}}*/

SoundSystem::SoundSystem()/*{{{*/
{
	for (int i=0; i<Sound::Dummy; i++)
		sample[i] = new QByteArray(0);
}/*}}}*/

SoundSystem::~SoundSystem()/*{{{*/
{
	for (int i=0; i<Sound::Dummy; i++)
		delete sample[i];
}/*}}}*/

bool SoundSystem::reload_samples(Settings &settings)/*{{{*/
{
	for (int i=0; i<Sound::Dummy; i++)
	{
		QFile f(settings.snd[i]); 
		if (f.open(IO_ReadOnly))
		{ 
			sample[i]->resize(f.size());
			f.readBlock(sample[i]->data(), f.size());
			f.close();
		}
	}
	return true;
}/*}}}*/

bool SoundPCSpeaker::init()/*{{{*/
{
	return true;
}/*}}}*/

void SoundPCSpeaker::play(Sound::Sfx sfx)/*{{{*/
{
	QApplication::beep();
}/*}}}*/

void SoundPCSpeaker::close()/*{{{*/
{
}/*}}}*/

/*
 * ARTS
 */
bool SoundArts::init()/*{{{*/
{
#	ifdef WITH_ARTS
	int err;
	if ((err = arts_init()) < 0)
	{
		errstr = arts_error_text(err);
		return false;
	}
#	else
	errstr = "no aRtS support compiled";
	return false;
#	endif
	return true;
}/*}}}*/

void SoundArts::play(Sound::Sfx sfx)/*{{{*/
{
#	ifdef WITH_ARTS
	arts_stream_t stream = arts_play_stream(22050, 8, 1, "vyqchat");
	int status = arts_write(stream, sample[sfx]->data(), sample[sfx]->size());
	if (status<0)
		qWarning("arts_write error: %s",  arts_error_text(status));
	arts_close_stream(stream);
#	endif
}/*}}}*/

void SoundArts::close()/*{{{*/
{
#	ifdef WITH_ARTS
	arts_free();
#	endif
}/*}}}*/

/*
 * ALSA
 */
bool SoundAlsa::init()/*{{{*/
{
	errstr = QString("Could not init Alsa. Alsa support is not ready yet");
	return false;
}/*}}}*/

void SoundAlsa::play(Sound::Sfx sfx)/*{{{*/
{
}/*}}}*/

void SoundAlsa::close()/*{{{*/
{
}/*}}}*/

/*
 * COMMAND
 */
bool SoundCommand::reload_samples(Settings &settings)/*{{{*/
{
	const QStringList cmd = QStringList::split(' ', settings.playcmd);
	for (int i=0; i<Sound::Dummy; i++)
	{
		arg[i].clear();
		arg[i] = cmd;
		arg[i].append(settings.snd[i]);
	}
	return true;
}/*}}}*/

bool SoundCommand::init()/*{{{*/
{
	return true;
}/*}}}*/

void SoundCommand::play(Sound::Sfx sfx)/*{{{*/
{
	QProcess *proc = new QProcess(arg[sfx], NULL);
	if (!proc->start())
		qWarning("cannot run play command");
}/*}}}*/

